/*
 * Test program for checking that all opened files are closed in case of
 * exception (especially for memory allocation exceptions).
 *
 * Author: Manlio Perillo
 */

#include <stdio.h>
#include <stdlib.h>

#include <lua.h>
#include <lauxlib.h>


#define MAX_ALLOC 10240

#define CHUNCK_1 "f = io.open(\"/usr/local/nginx/buf\");"
#define CHUNCK_2 "s = f:read(\"*a\");"



static void *l_alloc (void *ud, void *ptr, size_t osize, size_t nsize) {
  printf("l_alloc: %d:%d\n", osize, nsize);

  if (nsize == 0) {
    free(ptr);
    return NULL;
  }
  else if (nsize > MAX_ALLOC) {
    printf("l_alloc: out of memory\n");
    return NULL;
  }
  else {
    return realloc(ptr, nsize);
  }
}

int main (int argc, char **argv) {
  int status;
  lua_State *L = lua_newstate(l_alloc, NULL);

  if (L == NULL) {
    printf("cannot create state: not enough memory");
    return EXIT_FAILURE;
  }

  lua_gc(L, LUA_GCSTOP, 0);  /* stop collector during initialization */
  luaL_openlibs(L);  /* open standard libraries */
  lua_gc(L, LUA_GCRESTART, 0);

  /* 
   * Open a big file (> 100 KB) and read in memory its entire content.
   * Check if the file is closed with:
   *    lsof +d /usr/local/nginx
   */

  printf("press a key to start execution");
  getchar();

  status = luaL_dostring(L, CHUNCK_1);
  printf("execution status 1: %i\n", status);

  printf("press a key to continue execution");
  getchar();

  status = luaL_dostring(L, CHUNCK_2);
  printf("execution status 2: %i\n", status);

  printf("press a key to close Lua state");
  getchar();

  lua_close(L);

  printf("press a key to exit");
  getchar();

  return EXIT_SUCCESS;
}
